#ifndef _COMPAT_XEN_H
#define _COMPAT_XEN_H
#include <xen/compat.h>
#include <public/xen.h>
#pragma pack(4)
#include <public/xen-compat.h>
#pragma pack(4)

#include "arch-x86/xen.h"
#pragma pack(4)
struct compat_mmuext_op {
    unsigned int cmd;
    union {

        compat_pfn_t mfn;

        unsigned int linear_addr;
    } arg1;
    union {

        unsigned int nr_ents;

        COMPAT_HANDLE(void) vcpumask;
    } arg2;
};
typedef struct compat_mmuext_op mmuext_op_compat_t;
DEFINE_COMPAT_HANDLE(mmuext_op_compat_t);
typedef uint16_t domid_compat_t;
struct compat_mmu_update {
    uint64_t ptr;
    uint64_t val;
};
typedef struct mmu_update mmu_update_compat_t;
DEFINE_COMPAT_HANDLE(mmu_update_compat_t);

struct compat_multicall_entry {
    unsigned int op, result;
    unsigned int args[6];
};
typedef struct compat_multicall_entry multicall_entry_compat_t;
DEFINE_COMPAT_HANDLE(multicall_entry_compat_t);

struct compat_vcpu_time_info {
    uint32_t version;
    uint32_t pad0;
    uint64_t tsc_timestamp;
    uint64_t system_time;

    uint32_t tsc_to_system_mul;
    int8_t tsc_shift;
    int8_t pad1[3];
};
typedef struct vcpu_time_info vcpu_time_info_compat_t;

struct compat_vcpu_info {
    uint8_t evtchn_upcall_pending;
    uint8_t evtchn_upcall_mask;
    unsigned int evtchn_pending_sel;
    struct compat_arch_vcpu_info arch;
    struct vcpu_time_info time;
};
struct compat_shared_info {
    struct compat_vcpu_info vcpu_info[MAX_VIRT_CPUS];
    unsigned int evtchn_pending[sizeof(unsigned int) * 8];
    unsigned int evtchn_mask[sizeof(unsigned int) * 8];

    uint32_t wc_version;
    uint32_t wc_sec;
    uint32_t wc_nsec;

    struct compat_arch_shared_info arch;

};
struct compat_start_info {

    char magic[32];
    unsigned int nr_pages;
    unsigned int shared_info;
    uint32_t flags;
    compat_pfn_t store_mfn;
    uint32_t store_evtchn;
    union {
        struct {
            compat_pfn_t mfn;
            uint32_t evtchn;
        } domU;
        struct {
            uint32_t info_off;
            uint32_t info_size;
        } dom0;
    } console;

    unsigned int pt_base;
    unsigned int nr_pt_frames;
    unsigned int mfn_list;
    unsigned int mod_start;
    unsigned int mod_len;
    int8_t cmd_line[1024];
};
typedef struct compat_start_info start_info_compat_t;
typedef struct compat_dom0_vga_console_info {
    uint8_t video_type;

    union {
        struct {

            uint16_t font_height;

            uint16_t cursor_x, cursor_y;

            uint16_t rows, columns;
        } text_mode_3;

        struct {

            uint16_t width, height;

            uint16_t bytes_per_line;

            uint16_t bits_per_pixel;

            uint32_t lfb_base;
            uint32_t lfb_size;

            uint8_t red_pos, red_size;
            uint8_t green_pos, green_size;
            uint8_t blue_pos, blue_size;
            uint8_t rsvd_pos, rsvd_size;
        } vesa_lfb;
    } u;
} dom0_vga_console_info_compat_t;

typedef uint8_t compat_domain_handle_t[16];

DEFINE_COMPAT_HANDLE(uint8_t);
DEFINE_COMPAT_HANDLE(uint16_t);
DEFINE_COMPAT_HANDLE(uint32_t);
DEFINE_COMPAT_HANDLE(uint64_t);
#pragma pack()
#endif /* _COMPAT_XEN_H */
